Imports System.ComponentModel
Imports System.Drawing
Imports System.Windows.Forms
Imports Telerik.Reporting
Imports Telerik.Reporting.Drawing

Partial Public Class Report1
    Inherits Telerik.Reporting.Report
    Public Sub New()
        InitializeComponent()
    End Sub
End Class

<Expressions.AggregateFunction(Name:="CountDistinct", Description:="Defines an aggregate function to count distinct values.")> _
Public Class CountDistinctAggregate
    Implements Telerik.Reporting.Expressions.IAggregateFunction

    Private distinctValues As System.Collections.Generic.List(Of Object)

    ''' <summary>
    ''' Initializes the current aggregate function to its initial state
    ''' ready to accumulate and merge values. 
    ''' </summary>
    ''' <remarks>
    ''' This method is called every time the accumulation of values must
    ''' start over for a new subset of records from the data source.
    ''' </remarks>
    Sub Init() Implements Expressions.IAggregateFunction.Init
        Me.distinctValues = New System.Collections.Generic.List(Of Object)()
    End Sub

    ''' <summary>
    ''' Accumulates new argument values to the current aggregate function.
    ''' </summary>
    ''' <remarks>
    ''' This aggregate function accepts one argument:
    ''' number - a numeric value to accumulate to the aggregate function;
    ''' </remarks>
    Sub Accumulate(ByVal values() As Object) Implements Expressions.IAggregateFunction.Accumulate

        If Not distinctValues.Contains(values(0)) Then
            distinctValues.Add(values(0))
        End If
    End Sub


    ''' <summary>
    ''' Merges the specified aggregate function to the current one.
    ''' </summary>
    ''' <param name="Aggregate">
    ''' Specifies an aggregate function to be merged to the current one.
    ''' </param>
    ''' <remarks>
    ''' This method allows the reporting engine to merge two accumulated
    ''' subsets of the same aggregate function into a single result.
    ''' </remarks>
    Sub Merge(ByVal aggregate As Expressions.IAggregateFunction) Implements Expressions.IAggregateFunction.Merge
        ' Accumulate the values of the specified aggregate function.
        Dim sums1 = DirectCast(aggregate, CountDistinctAggregate).distinctValues
        For Each o As Object In sums1
            Me.Accumulate(New Object() {o})
        Next
    End Sub

    ''' <summary>
    ''' Returns the currently accumulated value of the aggregate function.
    ''' </summary>
    ''' <returns>
    ''' The currently accumulated numeric value of the aggregate function.
    ''' </returns>
    Function GetValue() As Object Implements Expressions.IAggregateFunction.GetValue
        Return Me.distinctValues.Count
    End Function
End Class
